/************************************************************/
/* class to evaluate the user defined helicity amplitude    */
/* the function here is taken from the "Z LINE SHAPE" paper */
/************************************************************/
#include "newZresonance.h"

using namespace std;

namespace Moses1 {
// this class inherit functions from the "couplings" class

newZresonance::newZresonance(string model) {
	
	m_prm.setParameters(m_physPrm); // initialize the new instance of the physical parameters struct
	m_prm.setParameters(m_runPrm); // initialize the new instance of the run parameters struct
	m_pdtPrt.setPDTtable("HEPPDTTBL","particle.tbl");

	m_print = false;

	if      (model == "chi") {
		m_model = 1;
		m_widthMassRatio = 0.012;
		m_theta = -1.*m_physPrm.pi/2.;
	}
	else if (model == "psi") {
		m_model = 2;
		m_widthMassRatio = 0.005;
		m_theta = 0.;
	}
	else if (model == "eta") {
		m_model = 3;
		m_widthMassRatio = 0.006;
		m_theta = atan(-1.*sqrt(5./3.)) + m_physPrm.pi/2.;
	}
	else {cerr << "in newZresonance.cxx: error - invalid option: model = " << model << endl;}

	setBosonPropagators();    // fill the structs with the PDT properties of gamma, Z0, Z'
	
	m_g2 = 0.461;
}

newZresonance::~newZresonance(){}



void newZresonance::setBosonPropagators() {
	m_pdtPrt.getAll(PDTGAMMA,   m_photonPropagator, m_print);
	m_pdtPrt.getAll(PDTZ,       m_Z0Propagator, m_print);
	m_pdtPrt.getAll(PDTZPRIME0, m_ZprimePropagator, m_print);

	m_propagators.push_back(m_photonPropagator);
	m_propagators.push_back(m_Z0Propagator);
	m_propagators.push_back(m_ZprimePropagator);
}



double newZresonance::getVAcouplings(customParticle &fermion, pdtParticle &propagator ,bool isLeft) {
	double couplings = 0;
	double LRsign = 1.;
	if (isLeft) { LRsign = +1.;}// gL ~ vf ?+? af
	else        { LRsign = -1.;}// gR ~ vf ?-? af

	if (propagator.id == PDTGAMMA) {couplings = fermion.pdt_prt.charge;} // SM photon

	else if (propagator.id == PDTZ) { // SM Z0
		couplings = ( (1./(2*sqrt(m_physPrm.sinThetaW2*m_physPrm.cosThetaW2))) *
			      ( fermion.weakI3 - 2.*fermion.pdt_prt.charge*m_physPrm.sinThetaW2 + LRsign*fermion.weakI3 )
			    );
	}

	else if (propagator.id == PDTZPRIME0) { // E6 Z'
		if (fermion.pdt_prt.id >= PDTE  &&  fermion.pdt_prt.id <= PDTNUTAU) { // leptons
			if (isLeft) {
				switch (m_model) {
					case 1: couplings = m_g2 * 3./(2.*sqrt(10)); break; // chi
					case 2: couplings = m_g2 * 1./(2.*sqrt(6)); break; // psi
					case 3: couplings = m_g2 * 1./(2.*sqrt(15)); break; // eta
					default : break;
				}
			}
			else {
				switch (m_model) {
					case 1: couplings = m_g2 * 1./(2.*sqrt(10)); break; // chi
					case 2: couplings = m_g2 * -1./(2.*sqrt(6)); break; // psi
					case 3: couplings = m_g2 * 1./(sqrt(15)); break; // eta
					default : break;
				}
			}
		}
		else if (fermion.pdt_prt.id>=PDTDWN && fermion.pdt_prt.id<=PDTTOP) {
			if (isLeft) {
				switch (m_model) {
					case 1: couplings = m_g2 * -1./(2.*sqrt(10)); break; // chi
					case 2: couplings = m_g2 * 1./(2.*sqrt(6)); break; // psi
					case 3: couplings = m_g2 * -1./(sqrt(15)); break; // eta
					default : break;
				}
			}
			else {
				if (fermion.pdt_prt.id==PDTDWN || fermion.pdt_prt.id==PDTSTR || fermion.pdt_prt.id==PDTBOT) {
					switch (m_model) {
						case 1: couplings = m_g2 * -3./(2.*sqrt(10)); break; // chi
						case 2: couplings = m_g2 * -1./(2.*sqrt(6)); break; // psi
						case 3: couplings = m_g2 * -1./(2.*sqrt(15)); break; // eta
						default : break;
					}
				}
				else if (fermion.pdt_prt.id==PDTUP || fermion.pdt_prt.id==PDTCHM || fermion.pdt_prt.id==PDTTOP) {
					switch (m_model) {
						case 1: couplings = m_g2 * 1./(2.*sqrt(10)); break; // chi
						case 2: couplings = m_g2 * -1./(2.*sqrt(6)); break; // psi
						case 3: couplings = m_g2 * 1./(sqrt(15)); break; // eta
						default : break;
					}
				}
			}
		}
	}

	else {cerr << "in newZresonance.cxx: error - invalid / unsupported propagator id = " << propagator.id << endl;}

	return(couplings);
}

/*
double newZresonance::Qtag_f(int fermionId, string LeftRight, string model) {
	double Q = 0; // the extra U(1)'  charge
	switch(fermionId) {
		case PDTDWN: //d
			if (LeftRight == "L") {
				switch (m_model) {
					case 1: Q = -1.; break; // chi
					case 2: Q = +1.; break; // psi
					case 3: Q = -2.; break; // eta
					default : break;
				}
			}
			else if (LeftRight == "R") {
				switch (m_model) {
					case 1: Q = +3.; break; // chi
					case 2: Q = +1.; break; // psi
					case 3: Q = +1.; break; // eta
					default : break;
				}
			}
			break;
		case PDTUP: //u
			if (LeftRight == "L") {
				switch (m_model) {
					case 1: Q = -1.; break; // chi
					case 2: Q = +1.; break; // psi
					case 3: Q = -2.; break; // eta
					default : break;
				}
			}
			else if (LeftRight == "R") {
				switch (m_model) {
					case 1: Q = -1.; break; // chi
					case 2: Q = +1.; break; // psi
					case 3: Q = -2.; break; // eta
					default : break;
				}
			}
			break;
		case PDTSTR: //s
			goto PDTDWN;
		case PDTCHM: //c
			goto PDTUP;
		case PDTBOT: //b
			goto PDTDWN;
		case PDTTOP: //t
			goto PDTUP;
		case PDTE: //e
			if (LeftRight == "L") {
				switch (m_model) {
					case 1: Q = +3.; break; // chi
					case 2: Q = +1.; break; // psi
					case 3: Q = +1.; break; // eta
					default : break;
				}
			}
			else if (LeftRight == "R") {
				switch (m_model) {
					case 1: Q = -1.; break; // chi
					case 2: Q = +1.; break; // psi
					case 3: Q = -2.; break; // eta
					default : break;
				}
			}
			break;			
		case PDTNUE: //nu_e
			goto PDTE;
		case PDTMU: //mu
			goto PDTE;
		case PDTNUMU: //nu_mu
			goto PDTE;
		case PDTTAU: //tau
			goto PDTE;
		case PDTNUTAU: //nu_tau
			goto PDTE;
		default: break;
	}
	return(Q);
}
*/

double newZresonance::getHelicityCouplings(double helicity, pdtParticle &propagator, customParticle &fermion) {
	
	double coup = 0;
	double sign = 1.;
	if      (helicity > 0) {sign = -1.;}
	else if (helicity < 0) {sign = +1.;}
	
	double gA = getAxialCouplings(propagator, fermion);
	double gV = getVectorCouplings(propagator, fermion);

	coup = ( gA + sign*gV ) / ( 2. * sqrt(m_physPrm.sinThetaW2 * m_physPrm.cosThetaW2) );

	return( coup );
}


double newZresonance::getAxialCouplings(pdtParticle &propagator, customParticle &fermion) {
	double coup = 0;

	if (fermion.pdt_prt.id == PDTNUE || fermion.pdt_prt.id == PDTNUMU || fermion.pdt_prt.id == PDTNUTAU) {
		coup = (1./6.)*sqrt(5./2.)*cos(m_theta) - (3./2.)*sqrt(1./6.)*sin(m_theta);
	}
	else if (fermion.pdt_prt.id == PDTE || fermion.pdt_prt.id == PDTMU || fermion.pdt_prt.id == PDTTAU) {
		coup = (1./3.)*sqrt(5./2.)*cos(m_theta) - sqrt(1./6.)*sin(m_theta);
	}
	else if (fermion.pdt_prt.id == PDTDWN || fermion.pdt_prt.id == PDTSTR || fermion.pdt_prt.id == PDTBOT) {
		coup = (1./3.)*sqrt(5./2.)*cos(m_theta) - sqrt(1./6.)*sin(m_theta);
	}
	else if (fermion.pdt_prt.id == PDTUP || fermion.pdt_prt.id == PDTCHM || fermion.pdt_prt.id == PDTTOP) {
		coup = (1./3.)*sqrt(5./2.)*cos(m_theta) + sqrt(1./6.)*sin(m_theta);
	}
	else {cerr << "error: in newZresonance.cxx//getAxialCouplings: unsupported fermion id" << endl;}

	return( coup );
}



double newZresonance::getVectorCouplings(pdtParticle &propagator, customParticle &fermion) {
	double coup = 0;

	if (fermion.pdt_prt.id == PDTNUE || fermion.pdt_prt.id == PDTNUMU || fermion.pdt_prt.id == PDTNUTAU) {
		coup = (1./6.)*sqrt(5./2.)*cos(m_theta) - (3./2.)*sqrt(1./6.)*sin(m_theta);
	}
	else if (fermion.pdt_prt.id == PDTE || fermion.pdt_prt.id == PDTMU || fermion.pdt_prt.id == PDTTAU) {
		coup = -2.*sqrt(1./6.)*sin(m_theta);
	}
	else if (fermion.pdt_prt.id == PDTDWN || fermion.pdt_prt.id == PDTSTR || fermion.pdt_prt.id == PDTBOT) {
		coup = +2.*sqrt(1./6.)*sin(m_theta);
	}
	else if (fermion.pdt_prt.id == PDTUP || fermion.pdt_prt.id == PDTCHM || fermion.pdt_prt.id == PDTTOP) {
		coup = 0;
	}
	else {cerr << "error: in newZresonance.cxx//getAxialCouplings: unsupported fermion id" << endl;}

	return( coup );
}



dcomplex newZresonance::getSquaredAmplitude(double hIn, double hOut, double s,
				customParticle &inFermion, customParticle &outFermion,
				double totalZ0width, double totalwidth_Zprime, bool doSMcouplings) {

	dcomplex helicityAmp (0,0);

	helicityAmp += m_couplings.getCouplings(hIn,m_photonPropagator,inFermion,m_physPrm)*m_couplings.getCouplings(hOut,m_photonPropagator,outFermion,m_physPrm)/s;
	helicityAmp += m_couplings.getCouplings(hIn,m_Z0Propagator,inFermion,m_physPrm)*m_couplings.getCouplings(hOut,m_Z0Propagator,outFermion,m_physPrm) /
			( s - m_Z0Propagator.mass*m_Z0Propagator.mass + m_physPrm.i * s * (totalZ0width/m_Z0Propagator.mass) );
	if (!doSMcouplings) {
		helicityAmp += getHelicityCouplings(hIn,m_ZprimePropagator,inFermion)*getHelicityCouplings(hOut,m_ZprimePropagator,outFermion) /
				( s - m_physPrm.mStar*m_physPrm.mStar + m_physPrm.i * s * totalwidth_Zprime/m_physPrm.mStar /*(m_widthMassRatio)*/ );
	}
	else {
		helicityAmp += m_couplings.getCouplings(hIn,m_Z0Propagator,inFermion,m_physPrm)*m_couplings.getCouplings(hOut,m_Z0Propagator,outFermion,m_physPrm) / ( s - m_physPrm.mStar*m_physPrm.mStar + m_physPrm.i * s * totalwidth_Zprime/m_physPrm.mStar /*(m_widthMassRatio)*/ );
	}
	return (helicityAmp);
}


dcomplex newZresonance::getSquaredAmplitude(double hIn, double hOut, double s, double mZprime,
                                customParticle &inFermion, customParticle &outFermion,
                                double totalZ0width, double totalwidth_Zprime, bool doSMcouplings) {

        dcomplex helicityAmp (0,0);

        helicityAmp += m_couplings.getCouplings(hIn,m_photonPropagator,inFermion,m_physPrm)*m_couplings.getCouplings(hOut,m_photonPropagator,outFermion,m_physPrm)/s;
        helicityAmp += m_couplings.getCouplings(hIn,m_Z0Propagator,inFermion,m_physPrm)*m_couplings.getCouplings(hOut,m_Z0Propagator,outFermion,m_physPrm) /
                        ( s - m_Z0Propagator.mass*m_Z0Propagator.mass + m_physPrm.i * s * (totalZ0width/m_Z0Propagator.mass) );
        if (!doSMcouplings) {
                helicityAmp += getHelicityCouplings(hIn,m_ZprimePropagator,inFermion)*getHelicityCouplings(hOut,m_ZprimePropagator,outFermion) /
                                ( s - mZprime*mZprime + m_physPrm.i * s * (totalwidth_Zprime/mZprime) /*(m_widthMassRatio)*/ );
        }
        else {
                helicityAmp += m_couplings.getCouplings(hIn,m_Z0Propagator,inFermion,m_physPrm)*m_couplings.getCouplings(hOut,m_Z0Propagator,outFermion,m_physPrm) / ( s - mZprime*mZprime + m_physPrm.i * s * (totalwidth_Zprime/mZprime) /*(m_widthMassRatio)*/ );
        }
        return (helicityAmp);
}



double newZresonance::getZprimeWidth(bool doSMcouplings, customParticle &fermion) {
	double width = 0;
	double gA = 0;
	double gV = 0;

	if (2*fermion.pdt_prt.mass >= m_physPrm.mStar) { width = 0; }

	if (!doSMcouplings) {
		gA = getVectorCouplings(m_ZprimePropagator,fermion);
		gV = getVectorCouplings(m_ZprimePropagator,fermion);
	}
	else {
		gA = (m_couplings.getCouplings(-0.5,m_Z0Propagator,fermion,m_physPrm) +
		      m_couplings.getCouplings(-0.5,m_Z0Propagator,fermion,m_physPrm)) * sqrt(m_physPrm.cosThetaW2*m_physPrm.sinThetaW2);
		gV = (m_couplings.getCouplings(-0.5,m_Z0Propagator,fermion,m_physPrm) -
		      m_couplings.getCouplings(-0.5,m_Z0Propagator,fermion,m_physPrm)) * sqrt(m_physPrm.cosThetaW2*m_physPrm.sinThetaW2);
	}
	width = ( m_physPrm.alpha / (12.*m_physPrm.cosThetaW2) ) * fermion.Nc * m_physPrm.mStar * ( gV*gV + gA*gA );
	return(width);
}



}







